@@ -13,11 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from os_brick.encryptors import cryptsetup
-from os_brick.privileged import rootwrap as priv_rootwrap
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 
+from os_brick.encryptors import cryptsetup
+from os_brick.privileged import rootwrap as priv_rootwrap
+
 LOG = logging.getLogger(__name__)
 
 
@@ -112,45 +113,6 @@ class LuksEncryptor(cryptsetup.CryptsetupEncryptor):
                       run_as_root=True, check_exit_code=True,
                       root_helper=self._root_helper)
 
-    def _unmangle_volume(self, key, passphrase, **kwargs):
-        """Workaround for bug#1633518
-
-        First identify if a mangled passphrase is used and if found then
-        replace with the correct unmangled version of the passphrase.
-        """
-        mangled_passphrase = self._get_mangled_passphrase(key)
-        self._open_volume(mangled_passphrase, **kwargs)
-        self._close_volume(**kwargs)
-        LOG.debug("%s correctly opened with a mangled passphrase, replacing "
-                  "this with the original passphrase", self.dev_path)
-
-        # NOTE(lyarwood): Now that we are sure that the mangled passphrase is
-        # used attempt to add the correct passphrase before removing the
-        # mangled version from the volume.
-
-        # luksAddKey currently prompts for the following input :
-        # Enter any existing passphrase:
-        # Enter new passphrase for key slot:
-        # Verify passphrase:
-        self._execute('cryptsetup', 'luksAddKey', self.dev_path,
-                      '--force-password',
-                      process_input=''.join([mangled_passphrase, '\n',
-                                             passphrase, '\n', passphrase]),
-                      run_as_root=True, check_exit_code=True,
-                      root_helper=self._root_helper)
-
-        # Verify that we can open the volume with the current passphrase
-        # before removing the mangled passphrase.
-        self._open_volume(passphrase, **kwargs)
-        self._close_volume(**kwargs)
-
-        # luksRemoveKey only prompts for the key to remove.
-        self._execute('cryptsetup', 'luksRemoveKey', self.dev_path,
-                      process_input=mangled_passphrase,
-                      run_as_root=True, check_exit_code=True,
-                      root_helper=self._root_helper)
-        LOG.debug("%s mangled passphrase successfully replaced", self.dev_path)
-
     def attach_volume(self, context, **kwargs):
         """Shadow the device and pass an unencrypted version to the instance.
 
@@ -175,16 +137,6 @@ class LuksEncryptor(cryptsetup.CryptsetupEncryptor):
                          self.dev_path)
                 self._format_volume(passphrase, **kwargs)
                 self._open_volume(passphrase, **kwargs)
-            elif e.exit_code == 2:
-                # NOTE(lyarwood): Workaround bug#1633518 by replacing any
-                # mangled passphrases that are found on the volume.
-                # TODO(lyarwood): Remove workaround during R.
-                LOG.warning("%s is not usable with the current "
-                            "passphrase, attempting to use a mangled "
-                            "passphrase to open the volume.",
-                            self.dev_path)
-                self._unmangle_volume(key, passphrase, **kwargs)
-                self._open_volume(passphrase, **kwargs)
             else:
                 raise
 
@@ -223,7 +175,7 @@ class Luks2Encryptor(LuksEncryptor):
             connection_info=connection_info,
             keymgr=keymgr,
             execute=execute,
-            *args, **kwargs)
+            *args, **kwargs)  # type: ignore
 
     def _format_volume(self, passphrase, **kwargs):
         """Creates a LUKS v2 header on the volume.
