@@ -474,7 +474,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             root_helper=self._root_helper)
 
     @utils.trace
-    @synchronized('extend_volume')
+    @synchronized('extend_volume', external=True)
     def extend_volume(self, connection_properties: dict):
         """Update the local kernel's size information.
 
@@ -497,18 +497,10 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             raise exception.VolumePathsNotFound()
 
     @utils.trace
-    @synchronized('connect_volume')
+    @synchronized('connect_volume', external=True)
     def connect_volume(self, connection_properties: dict):
         """Attach the volume to instance_name.
 
-        NOTE: Will retry up to three times to handle the case where c-vol
-        and n-cpu are both using os-brick to manage iSCSI sessions but they
-        are on the same node and using different locking directories. In this
-        case, even though this call is synchronized, they will be separate
-        locks and can still overlap with connect and disconnect. Since a
-        disconnect during an initial attach can't cause IO failure (the device
-        has not been made available yet), we just try the connection again.
-
         :param connection_properties: The valid dictionary that describes all
                                       of the target volume attributes.
         :type connection_properties: dict
@@ -870,7 +862,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         return device_map
 
     @utils.trace
-    @synchronized('connect_volume')
+    @synchronized('connect_volume', external=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Detach the volume from instance_name.
@@ -1051,6 +1043,17 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         return ips, iqns
 
     def _connect_to_iscsi_portal(self, connection_properties):
+        """Safely connect to iSCSI portal-target and return the session id."""
+        portal = connection_properties['target_portal'].split(",")[0]
+        target_iqn = connection_properties['target_iqn']
+
+        lock_name = f'connect_to_iscsi_portal-{portal}-{target_iqn}'
+        method = synchronized(
+            lock_name, external=True)(self._connect_to_iscsi_portal_unsafe)
+        return method(connection_properties)
+
+    @utils.retry((exception.BrickException))
+    def _connect_to_iscsi_portal_unsafe(self, connection_properties):
         """Connect to an iSCSI portal-target an return the session id."""
         portal = connection_properties['target_portal'].split(",")[0]
         target_iqn = connection_properties['target_iqn']
@@ -1066,9 +1069,17 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         out, err = self._run_iscsiadm(connection_properties, (),
                                       check_exit_code=(0, 21, 255))
         if err:
-            self._run_iscsiadm(connection_properties,
-                               ('--interface', self._get_transport(),
-                                '--op', 'new'))
+            out_new, err_new = self._run_iscsiadm(connection_properties,
+                                                  ('--interface',
+                                                   self._get_transport(),
+                                                   '--op', 'new'),
+                                                  check_exit_code=(0, 6))
+            if err_new:
+                # retry if iscsiadm returns 6 for "database failure"
+                LOG.debug("Retrying to connect to iSCSI portal %s", portal)
+                msg = (_("Encountered database failure for %s.") % (portal))
+                raise exception.BrickException(msg=msg)
+
         # Try to set the scan mode to manual
         res = self._iscsiadm_update(connection_properties,
                                     'node.session.scan', 'manual',
