@@ -10,6 +10,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import errno
 import glob
 import json
 import os.path
@@ -27,6 +28,7 @@ try:
     from os_brick.initiator.connectors import nvmeof_agent
 except ImportError:
     nvmeof_agent = None
+from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick import utils
 
 DEV_SEARCH_PATH = '/dev/'
@@ -62,11 +64,12 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         volume_replicas = connection_properties.get('volume_replicas')
         if not volume_replicas:  # compatibility
             return []
+        replica_count = connection_properties.get('replica_count')
 
         try:
-            if volume_replicas and len(volume_replicas) > 1:
+            if volume_replicas and replica_count > 1:
                 return ['/dev/md/' + connection_properties.get('alias')]
-            if volume_replicas and len(volume_replicas) == 1:
+            if volume_replicas and replica_count == 1:
                 return [NVMeOFConnector.get_nvme_device_path(
                     self, volume_replicas[0]['target_nqn'],
                     volume_replicas[0]['vol_uuid'])]
@@ -77,15 +80,31 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         except exception.VolumeDeviceNotFound:
             return []
 
-    @staticmethod
-    def get_connector_properties(root_helper, *args, **kwargs):
+    @classmethod
+    def nvme_present(cls):
+        try:
+            priv_rootwrap.custom_execute('nvme', 'version')
+            return True
+        except Exception as exc:
+            if isinstance(exc, OSError) and exc.errno == errno.ENOENT:
+                LOG.debug('nvme not present on system')
+            else:
+                LOG.warning('Unknown error when checking presence of nvme: %s',
+                            exc)
+        return False
+
+    @classmethod
+    def get_connector_properties(cls, root_helper, *args, **kwargs):
         """The NVMe-oF connector properties (initiator uuid and nqn.)"""
-        nvmf = NVMeOFConnector(root_helper=root_helper,
-                               execute=kwargs.get('execute'))
+        execute = kwargs.get('execute') or priv_rootwrap.execute
+        nvmf = NVMeOFConnector(root_helper=root_helper, execute=execute)
         ret = {}
+
+        nqn = None
         uuid = nvmf._get_host_uuid()
         suuid = nvmf._get_system_uuid()
-        nqn = nvmf._get_host_nqn()
+        if cls.nvme_present():
+            nqn = utils.get_host_nqn()
         if uuid:
             ret['uuid'] = uuid
         if suuid:
@@ -109,29 +128,6 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 "Process execution error in _get_host_uuid: %s" % str(e))
             return None
 
-    def _get_host_nqn(self):
-        host_nqn = None
-        try:
-            with open('/etc/nvme/hostnqn', 'r') as f:
-                host_nqn = f.read().strip()
-                f.close()
-        except IOError:
-            try:
-                self._execute(
-                    'mkdir', '-m', '755', '-p', '/etc/nvme',
-                    root_helper=self._root_helper, run_as_root=True)
-                out, err = self._execute(
-                    'nvme', 'gen-hostnqn', '|', 'tee', '/etc/nvme/hostnqn',
-                    root_helper=self._root_helper, run_as_root=True)
-                if out.strip():
-                    host_nqn = out.strip()
-                self._execute(
-                    'chmod', '644', '/etc/nvme/hostnqn',
-                    root_helper=self._root_helper, run_as_root=True)
-            except Exception as e:
-                LOG.warning("Could not generate host nqn: %s" % str(e))
-        return host_nqn
-
     def _get_system_uuid(self):
         # RSD requires system_uuid to let Cinder RSD Driver identify
         # Nova node for later RSD volume attachment.
@@ -180,7 +176,7 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                     "when running nvme list.")
             raise exception.CommandExecutionFailed(message=msg)
 
-    @utils.retry(exceptions=exception.VolumePathsNotFound)
+    @utils.retry(exception.VolumePathsNotFound)
     def _get_device_path(self, current_nvme_devices):
         all_nvme_devices = self._get_nvme_devices()
         LOG.debug("all_nvme_devices are %(all_nvme_devices)s",
@@ -188,12 +184,40 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         path = set(all_nvme_devices) - set(current_nvme_devices)
         if not path:
             raise exception.VolumePathsNotFound()
-        return list(path)
+        return list(path)[0]
 
-    @utils.retry(exceptions=putils.ProcessExecutionError)
+    @utils.retry(exception.VolumeDeviceNotFound)
+    def _get_device_path_by_nguid(self, nguid):
+        device_path = os.path.join(DEV_SEARCH_PATH,
+                                   'disk',
+                                   'by-id',
+                                   'nvme-eui.%s' % nguid)
+        LOG.debug("Try to retrieve symlink to %(device_path)s.",
+                  {"device_path": device_path})
+        try:
+            path, _err = self._execute('readlink',
+                                       '-e',
+                                       device_path,
+                                       run_as_root=True,
+                                       root_helper=self._root_helper)
+            if not path:
+                raise exception.VolumePathsNotFound()
+            return path.rstrip()
+        except putils.ProcessExecutionError as e:
+            LOG.exception(e)
+            raise exception.VolumeDeviceNotFound(device=device_path)
+
+    @utils.retry(putils.ProcessExecutionError)
     def _try_connect_nvme(self, cmd):
-        self._execute(*cmd, root_helper=self._root_helper,
-                      run_as_root=True)
+        try:
+            self._execute(*cmd, root_helper=self._root_helper,
+                          run_as_root=True)
+        except putils.ProcessExecutionError as e:
+            # Idempotent connection to target.
+            # Exit code 70 means that target is already connected.
+            if e.exit_code == 70:
+                return
+            raise
 
     def _get_nvme_subsys(self):
         # Example output:
@@ -222,12 +246,25 @@ class NVMeOFConnector(base.BaseLinuxConnector):
 
         return ret_val
 
-    @utils.retry(exceptions=exception.NotFound, retries=5)
+    @staticmethod
+    def _filter_nvme_devices(current_nvme_devices, nvme_controller):
+        LOG.debug("Filter NVMe devices belonging to controller "
+                  "%(nvme_controller)s.",
+                  {"nvme_controller": nvme_controller})
+        nvme_name_pattern = "/dev/%sn[0-9]+" % nvme_controller
+        nvme_devices_filtered = list(
+            filter(
+                lambda device: re.match(nvme_name_pattern, device),
+                current_nvme_devices
+            )
+        )
+        return nvme_devices_filtered
+
+    @utils.retry(exception.NotFound, retries=5)
     def _is_nvme_available(self, nvme_name):
-        nvme_name_pattern = "/dev/%sn[0-9]+" % nvme_name
-        for nvme_dev_name in self._get_nvme_devices():
-            if re.match(nvme_name_pattern, nvme_dev_name):
-                return True
+        current_nvme_devices = self._get_nvme_devices()
+        if self._filter_nvme_devices(current_nvme_devices, nvme_name):
+            return True
         else:
             LOG.error("Failed to find nvme device")
             raise exception.NotFound()
@@ -269,25 +306,6 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         # Wait until nvme will be available in kernel
         return self._is_nvme_available(nvme_name)
 
-    def _try_disconnect_volume(self, conn_nqn, ignore_errors=False):
-        cmd = [
-            'nvme',
-            'disconnect',
-            '-n',
-            conn_nqn]
-        try:
-            self._execute(
-                *cmd,
-                root_helper=self._root_helper,
-                run_as_root=True)
-
-        except putils.ProcessExecutionError:
-            LOG.error(
-                "Failed to disconnect from NVMe nqn "
-                "%(conn_nqn)s", {'conn_nqn': conn_nqn})
-            if not ignore_errors:
-                raise
-
     @utils.trace
     @synchronized('connect_volume', external=True)
     def connect_volume(self, connection_properties):
@@ -306,13 +324,13 @@ class NVMeOFConnector(base.BaseLinuxConnector):
             return self._connect_volume_replicated(connection_properties)
 
         current_nvme_devices = self._get_nvme_devices()
-
         device_info = {'type': 'block'}
         conn_nqn = connection_properties['nqn']
         target_portal = connection_properties['target_portal']
         port = connection_properties['target_port']
         nvme_transport_type = connection_properties['transport_type']
         host_nqn = connection_properties.get('host_nqn')
+        device_nguid = connection_properties.get('volume_nguid')
         cmd = [
             'nvme', 'connect',
             '-t', nvme_transport_type,
@@ -328,20 +346,27 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                                target_portal, port)
         except exception.NotFound:
             LOG.error("Waiting for nvme failed")
-            self._try_disconnect_volume(conn_nqn, True)
             raise exception.NotFound(message="nvme connect: NVMe device "
                                              "not found")
-        path = self._get_device_path(current_nvme_devices)
-        device_info['path'] = path[0]
+        if device_nguid:
+            path = self._get_device_path_by_nguid(device_nguid)
+        else:
+            path = self._get_device_path(current_nvme_devices)
+        device_info['path'] = path
         LOG.debug("NVMe device to be connected to is %(path)s",
-                  {'path': path[0]})
+                  {'path': path})
         return device_info
 
     @utils.trace
     @synchronized('connect_volume', external=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
-        """Detach and flush the volume.
+        """Flush the volume.
+
+        Disconnect of volumes happens on storage system side. Here we could
+        remove connections to subsystems if no volumes are left. But new
+        volumes can pop up asynchronously in the meantime. So the only thing
+        left is flushing or disassembly of a correspondng RAID device.
 
         :param connection_properties: The dictionary that describes all
                                       of the target volume attributes.
@@ -370,32 +395,11 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                         {'device_path': device_path, 'conn_nqn': conn_nqn})
             return
 
-        exc = exception.ExceptionChainer()
-        with exc.context(force, 'Flushing %s failed', device_path):
+        try:
             self._linuxscsi.flush_device_io(device_path)
-
-        LOG.debug(
-            "Trying to disconnect from device %(device_path)s with "
-            "subnqn %(conn_nqn)s",
-            {'device_path': device_path, 'conn_nqn': conn_nqn})
-        cmd = [
-            'nvme',
-            'disconnect',
-            '-n',
-            conn_nqn]
-        with exc.context(force, "Failed to disconnect from NVMe nqn "
-                         "%(conn_nqn)s with device_path %(device_path)s",
-                         {'conn_nqn': conn_nqn, 'device_path': device_path}):
-            self._execute(
-                *cmd,
-                root_helper=self._root_helper,
-                run_as_root=True)
-
-        if exc:
-            LOG.warning('There were errors removing %s, leftovers may remain '
-                        'in the system', device_path)
+        except putils.ProcessExecutionError:
             if not ignore_errors:
-                raise exc
+                raise
 
     @utils.trace
     @synchronized('extend_volume', external=True)
@@ -407,9 +411,11 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         """
         if connection_properties.get('vol_uuid'):  # compatibility
             return self._extend_volume_replicated(connection_properties)
-
         volume_paths = self.get_volume_paths(connection_properties)
         if volume_paths:
+            if connection_properties.get('volume_nguid'):
+                for path in volume_paths:
+                    return self._linuxscsi.get_device_size(path)
             return self._linuxscsi.extend_volume(
                 volume_paths, use_multipath=self.use_multipath)
         else:
@@ -429,6 +435,7 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         """
 
         volume_replicas = connection_properties.get('volume_replicas')
+        replica_count = connection_properties.get('replica_count')
         volume_alias = connection_properties.get('alias')
 
         if volume_replicas:
@@ -447,9 +454,9 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 raise exception.VolumeDeviceNotFound(
                     device=volume_replicas)
 
-            if len(volume_replicas) > 1:
+            if replica_count > 1:
                 device_path = self._handle_replicated_volume(
-                    host_device_paths, volume_alias, len(volume_replicas))
+                    host_device_paths, volume_alias, replica_count)
             else:
                 device_path = self._handle_single_replica(
                     host_device_paths, volume_alias)
@@ -469,14 +476,15 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                                       force=False, ignore_errors=False):
         device_path = None
         volume_replicas = connection_properties.get('volume_replicas')
+        replica_count = connection_properties.get('replica_count')
         if device_info and device_info.get('path'):
             device_path = device_info['path']
         elif connection_properties.get('device_path'):
             device_path = connection_properties['device_path']
-        elif volume_replicas and len(volume_replicas) > 1:
+        elif volume_replicas and replica_count > 1:
             device_path = '/dev/md/' + connection_properties['alias']
 
-        if volume_replicas and len(volume_replicas) > 1:
+        if volume_replicas and replica_count > 1:
             NVMeOFConnector.end_raid(self, device_path)
         else:
             if self._get_fs_type(device_path) == 'linux_raid_member':
@@ -484,8 +492,9 @@ class NVMeOFConnector(base.BaseLinuxConnector):
 
     def _extend_volume_replicated(self, connection_properties):
         volume_replicas = connection_properties.get('volume_replicas')
+        replica_count = connection_properties.get('replica_count')
 
-        if volume_replicas and len(volume_replicas) > 1:
+        if volume_replicas and replica_count > 1:
             device_path = '/dev/md/' + connection_properties['alias']
             NVMeOFConnector.run_mdadm(
                 self, ['mdadm', '--grow', '--size', 'max', device_path])
@@ -503,33 +512,19 @@ class NVMeOFConnector(base.BaseLinuxConnector):
 
     def _connect_target_volume(self, target_nqn, vol_uuid, portals):
         try:
-            host_device_path = NVMeOFConnector.get_nvme_device_path(
-                self, target_nqn, vol_uuid)
-        except exception.VolumeDeviceNotFound:
-            host_device_path = None
-
-        if not host_device_path:
-            any_connect = NVMeOFConnector.connect_to_portals(
-                self, target_nqn, portals)
-            if not any_connect:
-                LOG.error(
-                    "No successful connections: %(host_devices)s",
-                    {'host_devices': target_nqn})
-                raise exception.VolumeDeviceNotFound(device=target_nqn)
-
-            host_device_path = NVMeOFConnector.get_nvme_device_path(
-                self, target_nqn, vol_uuid)
-            if not host_device_path:
-                LOG.error(
-                    "No accessible volume device: %(host_devices)s",
-                    {'host_devices': target_nqn})
-                raise exception.VolumeDeviceNotFound(device=target_nqn)
-        else:
+            NVMeOFConnector._get_nvme_controller(self, target_nqn)
             NVMeOFConnector.rescan(self, target_nqn, vol_uuid)
-            host_device_path = NVMeOFConnector.get_nvme_device_path(
-                self, target_nqn, vol_uuid)
-
-        return host_device_path
+        except exception.VolumeDeviceNotFound:
+            if not NVMeOFConnector.connect_to_portals(
+                    self, target_nqn, portals):
+                LOG.error("No successful connections to: %s", target_nqn)
+                raise exception.VolumeDeviceNotFound(device=target_nqn)
+        dev_path = NVMeOFConnector.get_nvme_device_path(
+            self, target_nqn, vol_uuid)
+        if not dev_path:
+            LOG.error("Target %s volume %s not found", target_nqn, vol_uuid)
+            raise exception.VolumeDeviceNotFound(device=vol_uuid)
+        return dev_path
 
     @staticmethod
     def connect_to_portals(executor, target_nqn, target_portals):
@@ -576,22 +571,21 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         raise exception.VolumeDeviceNotFound(device=target_nqn)
 
     @staticmethod
-    @utils.retry(exceptions=exception.VolumeDeviceNotFound)
+    @utils.retry(exception.VolumeDeviceNotFound, retries=5)
     def get_nvme_device_path(executor, target_nqn, vol_uuid):
         nvme_ctrl = NVMeOFConnector._get_nvme_controller(executor, target_nqn)
-        try:
-            blocks = glob.glob(
-                '/sys/class/nvme-fabrics/ctl/' + nvme_ctrl +
-                '/' + nvme_ctrl + 'n*')
-            for block in blocks:
+        uuid_paths = glob.glob('/sys/class/block/' + nvme_ctrl + 'n*/uuid')
+        for uuid_path in uuid_paths:
+            try:
                 uuid_lines, _err = executor._execute(
-                    'cat', block + '/uuid', run_as_root=True,
+                    'cat', uuid_path, run_as_root=True,
                     root_helper=executor._root_helper)
                 if uuid_lines.split('\n')[0] == vol_uuid:
-                    return '/dev/' + block[block.rfind('/') + 1:]
-        except putils.ProcessExecutionError as e:
-            LOG.exception(e)
-
+                    ignore = len('/uuid')
+                    return '/dev/' + uuid_path[
+                        uuid_path.rfind('/', 0, -ignore) + 1: -ignore]
+            except putils.ProcessExecutionError as e:
+                LOG.exception(e)
         raise exception.VolumeDeviceNotFound(device=vol_uuid)
 
     def _handle_replicated_volume(self, host_device_paths,
@@ -764,6 +758,19 @@ class NVMeOFConnector(base.BaseLinuxConnector):
 
         LOG.debug('[!] cmd = ' + str(cmd))
         NVMeOFConnector.run_mdadm(executor, cmd)
+        # sometimes under load, md is not created right away so we wait
+        for i in range(60):
+            try:
+                is_exist = os.path.exists("/dev/md/" + name)
+                LOG.debug("[!] md is_exist = %s", is_exist)
+                if is_exist:
+                    return
+                time.sleep(1)
+            except Exception:
+                LOG.debug('[!] Exception_wait_raid!')
+        msg = _("md: /dev/md/%s not found.") % name
+        LOG.error(msg)
+        raise exception.NotFound(message=msg)
 
     @staticmethod
     def end_raid(executor, device_path):
@@ -772,12 +779,11 @@ class NVMeOFConnector(base.BaseLinuxConnector):
             for i in range(10):
                 try:
                     cmd_out = NVMeOFConnector.stop_raid(
-                        executor, device_path)
+                        executor, device_path, True)
                     if not cmd_out:
                         break
                 except Exception:
-                    break
-                time.sleep(1)
+                    time.sleep(1)
             try:
                 is_exist = os.path.exists(device_path)
                 LOG.debug("[!] is_exist = %s", is_exist)
@@ -788,10 +794,10 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 LOG.debug('[!] Exception_stop_raid!')
 
     @staticmethod
-    def stop_raid(executor, md_path):
+    def stop_raid(executor, md_path, raise_exception=False):
         cmd = ['mdadm', '--stop', md_path]
         LOG.debug("[!] cmd = " + str(cmd))
-        cmd_out = NVMeOFConnector.run_mdadm(executor, cmd)
+        cmd_out = NVMeOFConnector.run_mdadm(executor, cmd, raise_exception)
         return cmd_out
 
     @staticmethod
