@@ -75,6 +75,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self.volume_path = None
         self.iops_limit = None
         self.bandwidth_limit = None
+        self.verify_certificate = None
+        self.certificate_path = None
 
     def _get_guid(self):
         try:
@@ -204,7 +206,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         r = requests.get(
             request,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request)
@@ -250,7 +252,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
         r = requests.get(request,
                          auth=(self.server_username, self.server_token),
-                         verify=False)
+                         verify=self._verify_cert())
 
         r = self._check_response(r, request)
 
@@ -290,7 +292,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             r = requests.get(
                 login_request,
                 auth=(self.server_username, self.server_password),
-                verify=False
+                verify=self._verify_cert()
             )
 
             token = r.json()
@@ -301,7 +303,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             if is_get_request:
                 res = requests.get(request,
                                    auth=(self.server_username, token),
-                                   verify=False)
+                                   verify=self._verify_cert())
             else:
                 headers = {'content-type': 'application/json'}
                 res = requests.post(
@@ -309,7 +311,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                     data=json.dumps(params),
                     headers=headers,
                     auth=(self.server_username, token),
-                    verify=False
+                    verify=self._verify_cert()
                 )
 
             self.server_token = token
@@ -317,6 +319,12 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
         return response
 
+    def _verify_cert(self):
+        verify_cert = self.verify_certificate
+        if self.verify_certificate and self.certificate_path:
+            verify_cert = self.certificate_path
+        return verify_cert
+
     def get_config(self, connection_properties):
         self.local_sdc_ip = connection_properties['hostIP']
         self.volume_name = connection_properties['scaleIO_volname']
@@ -331,12 +339,15 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             connection_properties)
         self.iops_limit = connection_properties['iopsLimit']
         self.bandwidth_limit = connection_properties['bandwidthLimit']
+        self.verify_certificate = (
+            connection_properties.get('verify_certificate')
+        )
+        self.certificate_path = connection_properties.get('certificate_path')
         device_info = {'type': 'block',
                        'path': self.volume_path}
         return device_info
 
     @utils.trace
-    @utils.connect_volume_prepare_result
     @lockutils.synchronized('scaleio', 'scaleio-', external=True)
     def connect_volume(self, connection_properties):
         """Connect the volume.
@@ -383,7 +394,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             data=json.dumps(params),
             headers=headers,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request, False, params)
@@ -432,7 +443,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                 data=json.dumps(params),
                 headers=headers,
                 auth=(self.server_username, self.server_token),
-                verify=False
+                verify=self._verify_cert()
             )
             r = self._check_response(r, request, False, params)
             if r.status_code != self.OK_STATUS_CODE:
@@ -452,7 +463,6 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
     @utils.trace
     @lockutils.synchronized('scaleio', 'scaleio-', external=True)
-    @utils.connect_volume_undo_prepare_result(unlink_after=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Disconnect the ScaleIO volume.
@@ -497,7 +507,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             data=json.dumps(params),
             headers=headers,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request, False, params)
@@ -516,7 +526,6 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                 LOG.error(msg)
                 raise exception.BrickException(message=msg)
 
-    @utils.connect_volume_undo_prepare_result
     def extend_volume(self, connection_properties):
         """Update the local kernel's size information.
 
