@@ -15,15 +15,12 @@
 
 import abc
 
-import six
-
 from os_brick import exception
 from os_brick import executor
 from os_brick import initiator
 
 
-@six.add_metaclass(abc.ABCMeta)
-class InitiatorConnector(executor.Executor):
+class InitiatorConnector(executor.Executor, metaclass=abc.ABCMeta):
 
     # This object can be used on any platform (x86, S390)
     platform = initiator.PLATFORM_ALL
@@ -42,6 +39,7 @@ class InitiatorConnector(executor.Executor):
         """The driver is used to find used LUNs."""
         self.driver = driver
 
+    @staticmethod
     @abc.abstractmethod
     def get_connector_properties(root_helper, *args, **kwargs):
         """The generic connector properties."""
@@ -85,13 +83,12 @@ class InitiatorConnector(executor.Executor):
              'access_mode': 'rw',
         }}
 
-        An example for fibre_channel:
+        An example for fibre_channel with single lun:
 
         {'driver_volume_type': 'fibre_channel',
          'data': {
-            'initiator_target_map': {'100010604b010459': ['21230002AC00383D'],
-                                     '100010604b01045d': ['21230002AC00383D']
-                                    },
+            'initiator_target_map': {'100010604b010459': ['20210002AC00383D'],
+                                     '100010604b01045d': ['20220002AC00383D']},
             'target_discovered': True,
             'encrypted': False,
             'qos_specs': None,
@@ -103,6 +100,31 @@ class InitiatorConnector(executor.Executor):
                 ],
          }}
 
+        An example for fibre_channel target_wwns and with different LUNs and
+        all host ports mapped to target ports:
+
+        {'driver_volume_type': 'fibre_channel',
+         'data': {
+            'initiator_target_map': {
+                '100010604b010459': ['20210002AC00383D', '20220002AC00383D'],
+                '100010604b01045d': ['20210002AC00383D', '20220002AC00383D']
+                },
+            'target_discovered': True,
+            'encrypted': False,
+            'qos_specs': None,
+            'target_luns': [1, 2],
+            'access_mode': 'rw',
+            'target_wwns': ['20210002AC00383D', '20220002AC00383D'],
+         }}
+
+         For FC the dictionary could also present the enable_wildcard_scan key
+         with a boolean value (defaults to True) in case a driver doesn't want
+         OS-Brick to use a SCSI scan with wildcards when the FC initiator on
+         the host doesn't find any target port.
+
+         This is useful for drivers that know that sysfs gets populated
+         whenever there's a connection between the host's HBA and the storage
+         array's target ports.
 
         :param connection_properties: The dictionary that describes all
                                       of the target volume attributes.
