@@ -13,13 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import binascii
-import mock
+from unittest import mock
+
+from oslo_concurrency import processutils as putils
 
-from castellan.common.objects import symmetric_key as key
 from os_brick.encryptors import luks
 from os_brick.tests.encryptors import test_cryptsetup
-from oslo_concurrency import processutils as putils
 
 
 class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
@@ -183,78 +182,6 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       attempts=3, run_as_root=True, check_exit_code=[0, 4]),
         ])
 
-    def test_get_mangled_passphrase(self):
-        # Confirm that a mangled passphrase is provided as per bug#1633518
-        unmangled_raw_key = bytes(binascii.unhexlify('0725230b'))
-        symmetric_key = key.SymmetricKey('AES', len(unmangled_raw_key) * 8,
-                                         unmangled_raw_key)
-        unmangled_encoded_key = symmetric_key.get_encoded()
-        self.assertEqual(self.encryptor._get_mangled_passphrase(
-            unmangled_encoded_key), '72523b')
-
-    @mock.patch('os_brick.executor.Executor._execute')
-    def test_attach_volume_unmangle_passphrase(self, mock_execute):
-        fake_key = '0725230b'
-        fake_key_mangled = '72523b'
-        self.encryptor._get_key = mock.MagicMock()
-        self.encryptor._get_key.return_value = \
-            test_cryptsetup.fake__get_key(None, fake_key)
-
-        mock_execute.side_effect = [
-            putils.ProcessExecutionError(exit_code=2),  # luksOpen
-            mock.DEFAULT,  # luksOpen
-            mock.DEFAULT,  # luksClose
-            mock.DEFAULT,  # luksAddKey
-            mock.DEFAULT,  # luksOpen
-            mock.DEFAULT,  # luksClose
-            mock.DEFAULT,  # luksRemoveKey
-            mock.DEFAULT,  # luksOpen
-            mock.DEFAULT,  # ln
-        ]
-
-        self.encryptor.attach_volume(None)
-
-        mock_execute.assert_has_calls([
-            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
-                      self.dev_name, process_input=fake_key,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
-                      self.dev_name, process_input=fake_key_mangled,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-            mock.call('cryptsetup', 'luksClose', self.dev_name,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=[0, 4], attempts=3),
-            mock.call('cryptsetup', 'luksAddKey', self.dev_path,
-                      '--force-password',
-                      process_input=''.join([fake_key_mangled,
-                                             '\n', fake_key,
-                                             '\n', fake_key]),
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
-                      self.dev_name, process_input=fake_key,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-            mock.call('cryptsetup', 'luksClose', self.dev_name,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=[0, 4], attempts=3),
-            mock.call('cryptsetup', 'luksRemoveKey', self.dev_path,
-                      process_input=fake_key_mangled,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
-                      self.dev_name, process_input=fake_key,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-            mock.call('ln', '--symbolic', '--force',
-                      '/dev/mapper/%s' % self.dev_name, self.symlink_path,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-        ], any_order=False)
-        self.assertEqual(9, mock_execute.call_count)
-
 
 class Luks2EncryptorTestCase(LuksEncryptorTestCase):
     def _create(self):
