@@ -24,6 +24,7 @@ from os_brick.initiator.connectors import nvmeof
 from os_brick.initiator import linuxscsi
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick.tests.initiator import test_connector
+from os_brick import utils
 
 
 TARGET_NQN = 'target.nqn'
@@ -45,7 +46,8 @@ volume_replicas = [{'target_nqn': 'fakenqn1', 'vol_uuid': 'fakeuuid1',
 connection_properties = {
     'alias': 'fakealias',
     'vol_uuid': 'fakevoluuid',
-    'volume_replicas': volume_replicas
+    'volume_replicas': volume_replicas,
+    'replica_count': 3
 }
 fake_portal = ('fake', 'portal', 'tcp')
 
@@ -137,7 +139,7 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
 
     @mock.patch.object(nvmeof.NVMeOFConnector, 'nvme_present',
                        return_value=True)
-    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_host_nqn',
+    @mock.patch.object(utils, 'get_host_nqn',
                        return_value='fakenqn')
     @mock.patch.object(nvmeof.NVMeOFConnector, '_get_system_uuid',
                        return_value=None)
@@ -151,7 +153,7 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         self.assertEqual(expected_props, props)
 
     @mock.patch.object(nvmeof.NVMeOFConnector, 'nvme_present')
-    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_host_nqn', autospec=True)
+    @mock.patch.object(utils, 'get_host_nqn', autospec=True)
     @mock.patch.object(nvmeof.NVMeOFConnector, '_get_system_uuid',
                        autospec=True)
     @mock.patch.object(nvmeof.NVMeOFConnector, '_get_host_uuid', autospec=True)
@@ -177,7 +179,8 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         mock_get_device_path.return_value = '/dev/nvme1n1'
         connection_properties = {
             'alias': 'fakealias',
-            'volume_replicas': [volume_replicas[0]]
+            'volume_replicas': [volume_replicas[0]],
+            'replica_count': 1
         }
         self.assertEqual(self.connector.get_volume_paths(
             connection_properties),
@@ -214,6 +217,23 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
                                              root_helper=None,
                                              run_as_root=True)
 
+    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_nvme_devices')
+    def test__get_device_path(self, mock_nvme_devices):
+        mock_nvme_devices.return_value = ['/dev/nvme0n1',
+                                          '/dev/nvme1n1',
+                                          '/dev/nvme0n2']
+        current_devices = ['/dev/nvme0n1', '/dev/nvme0n2']
+        self.assertEqual(self.connector._get_device_path(current_devices),
+                         '/dev/nvme1n1')
+
+    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_nvme_devices')
+    def test__get_device_path_no_new_device(self, mock_nvme_devices):
+        current_devices = ['/dev/nvme0n1', '/dev/nvme0n2']
+        mock_nvme_devices.return_value = current_devices
+        self.assertRaises(exception.VolumePathsNotFound,
+                          self.connector._get_device_path,
+                          current_devices)
+
     @mock.patch.object(nvmeof.NVMeOFConnector, '_execute', autospec=True)
     def test__get_device_path_by_nguid(self, mock_execute):
         mock_execute.return_value = '/dev/nvme0n1\n', None
@@ -240,7 +260,8 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         connection_properties1 = {
             'target_nqn': 'fakenqn',
             'vol_uuid': 'fakeuuid',
-            'volume_replicas': [volume_replicas[0]]
+            'volume_replicas': [volume_replicas[0]],
+            'replica_count': 1
         }
         mock_connect_target_volume.return_value = '/dev/nvme0n1'
         self.assertEqual(
@@ -384,6 +405,7 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         connection_properties = {
             'vol_uuid': 'fakeuuid',
             'volume_replicas': volume_replicas,
+            'replica_count': 3,
             'device_path': '/dev/md/md1'
         }
         self.connector.disconnect_volume(connection_properties, None)
@@ -414,7 +436,8 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         connection_properties = {
             'target_nqn': 'fakenqn',
             'vol_uuid': 'fakeuuid',
-            'volume_replicas': [volume_replicas[0]]
+            'volume_replicas': [volume_replicas[0]],
+            'replica_count': 1
         }
         mock_device_path.return_value = '/dev/nvme0n1'
         mock_device_size.return_value = 100
@@ -470,15 +493,18 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         )
         mock_device_size.assert_called_with(device_path)
 
+    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_nvme_controller')
     @mock.patch.object(nvmeof.NVMeOFConnector, 'rescan')
     @mock.patch.object(nvmeof.NVMeOFConnector, 'get_nvme_device_path')
     def test__connect_target_volume_with_connected_device(
-            self, mock_device_path, mock_rescan):
+            self, mock_device_path, mock_rescan, mock_controller):
         mock_device_path.return_value = '/dev/nvme0n1'
         self.assertEqual(
             self.connector._connect_target_volume(
                 'fakenqn', 'fakeuuid', [('fake', 'portal', 'tcp')]),
             '/dev/nvme0n1')
+        mock_controller.assert_called_with(self.connector, 'fakenqn')
+        mock_rescan.assert_called_with(self.connector, 'fakenqn', 'fakeuuid')
         mock_device_path.assert_called_with(
             self.connector, 'fakenqn', 'fakeuuid')
 
@@ -494,23 +520,23 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         mock_device_path.assert_called_with(
             self.connector, TARGET_NQN, VOL_UUID)
 
+    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_nvme_controller')
     @mock.patch.object(nvmeof.NVMeOFConnector, 'connect_to_portals')
-    @mock.patch.object(nvmeof.NVMeOFConnector, 'get_nvme_device_path')
     def test__connect_target_volume_no_portals_con(
-            self, mock_device_path, mock_portals):
-        mock_device_path.return_value = None
+            self, mock_portals, mock_controller):
+        mock_controller.side_effect = exception.VolumeDeviceNotFound()
         mock_portals.return_value = None
         self.assertRaises(exception.VolumeDeviceNotFound,
                           self.connector._connect_target_volume, 'fakenqn',
                           'fakeuuid', [fake_portal])
-        mock_device_path.assert_called_with(
-            self.connector, 'fakenqn', 'fakeuuid')
 
+    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_nvme_controller')
     @mock.patch.object(nvmeof.NVMeOFConnector, 'connect_to_portals')
     @mock.patch.object(nvmeof.NVMeOFConnector, 'get_nvme_device_path')
     def test__connect_target_volume_new_device_path(
-            self, mock_device_path, mock_connect_portal):
-        mock_device_path.side_effect = (None, '/dev/nvme0n1')
+            self, mock_device_path, mock_connect_portal, mock_controller):
+        mock_controller.side_effect = exception.VolumeDeviceNotFound()
+        mock_device_path.return_value = '/dev/nvme0n1'
         self.assertEqual(
             self.connector._connect_target_volume(
                 'fakenqn', 'fakeuuid', [('fake', 'portal', 'tcp')]),
@@ -644,8 +670,10 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
             ['mdadm', '--assemble', '--run', '/dev/md/md1', '-o', '/dev/sda'],
             True)
 
+    @mock.patch.object(os.path, 'exists')
     @mock.patch.object(nvmeof.NVMeOFConnector, 'run_mdadm')
-    def test_create_raid_cmd_simple(self, mock_run_mdadm):
+    def test_create_raid_cmd_simple(self, mock_run_mdadm, mock_os):
+        mock_os.return_value = True
         self.assertIsNone(self.connector.create_raid(
             self.connector, ['/dev/sda'], '1', 'md1', 'name', True))
         mock_run_mdadm.assert_called_with(
@@ -653,6 +681,7 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
             ['mdadm', '-C', '-o', 'md1', '-R', '-N', 'name', '--level', '1',
              '--raid-devices=1', '--bitmap=internal', '--homehost=any',
              '--failfast', '--assume-clean', '/dev/sda'])
+        mock_os.assert_called_with('/dev/md/name')
 
     @mock.patch.object(nvmeof.NVMeOFConnector, 'stop_raid')
     @mock.patch.object(nvmeof.NVMeOFConnector, 'is_raid_exists')
@@ -662,7 +691,7 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         self.assertIsNone(self.connector.end_raid(
             self.connector, '/dev/md/md1'))
         mock_raid_exists.assert_called_with(self.connector, '/dev/md/md1')
-        mock_stop_raid.assert_called_with(self.connector, '/dev/md/md1')
+        mock_stop_raid.assert_called_with(self.connector, '/dev/md/md1', True)
 
     @mock.patch.object(os.path, 'exists')
     @mock.patch.object(nvmeof.NVMeOFConnector, 'stop_raid')
@@ -674,7 +703,7 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         self.assertIsNone(self.connector.end_raid(
             self.connector, '/dev/md/md1'))
         mock_raid_exists.assert_called_with(self.connector, '/dev/md/md1')
-        mock_stop_raid.assert_called_with(self.connector, '/dev/md/md1')
+        mock_stop_raid.assert_called_with(self.connector, '/dev/md/md1', True)
         mock_os.assert_called_with('/dev/md/md1')
 
     @mock.patch.object(os.path, 'exists')
@@ -687,16 +716,16 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         self.assertIsNone(self.connector.end_raid(
             self.connector, '/dev/md/md1'))
         mock_raid_exists.assert_called_with(self.connector, '/dev/md/md1')
-        mock_stop_raid.assert_called_with(self.connector, '/dev/md/md1')
+        mock_stop_raid.assert_called_with(self.connector, '/dev/md/md1', True)
         mock_os.assert_called_with('/dev/md/md1')
 
     @mock.patch.object(nvmeof.NVMeOFConnector, 'run_mdadm')
     def test_stop_raid_simple(self, mock_run_mdadm):
         mock_run_mdadm.return_value = 'mdadm output'
         self.assertEqual(self.connector.stop_raid(
-            self.connector, '/dev/md/md1'), 'mdadm output')
+            self.connector, '/dev/md/md1', True), 'mdadm output')
         mock_run_mdadm.assert_called_with(
-            self.connector, ['mdadm', '--stop', '/dev/md/md1'])
+            self.connector, ['mdadm', '--stop', '/dev/md/md1'], True)
 
     @mock.patch.object(nvmeof.NVMeOFConnector, 'run_mdadm')
     def test_remove_raid_simple(self, mock_run_mdadm):
@@ -849,10 +878,10 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
     def test_get_nvme_device_path(self, mock_get_nvme_controller, mock_glob,
                                   mock_execute):
         mock_get_nvme_controller.return_value = 'nvme1'
-        block_dev_path = '/sys/class/nvme-fabrics/ctl/nvme1/nvme1n*'
-        mock_glob.side_effect = [['/sys/class/nvme-fabrics/ctl/nvme1/nvme1n1']]
+        block_dev_path = '/sys/class/block/nvme1n*/uuid'
+        mock_glob.return_value = ['/sys/class/block/nvme1n1/uuid']
         mock_execute.return_value = (VOL_UUID + "\n", "")
-        cmd = ['cat', '/sys/class/nvme-fabrics/ctl/nvme1/nvme1n1/uuid']
+        cmd = ['cat', '/sys/class/block/nvme1n1/uuid']
         result = self.connector.get_nvme_device_path(EXECUTOR, TARGET_NQN,
                                                      VOL_UUID)
         mock_get_nvme_controller.assert_called_with(EXECUTOR, TARGET_NQN)
@@ -931,21 +960,21 @@ class NVMeOFConnectorTestCase(test_connector.ConnectorTestCase):
         mock_open.assert_called_once_with('/etc/nvme/hostnqn', 'r')
         self.assertEqual(HOST_NQN, host_nqn)
 
-    @mock.patch.object(nvmeof.priv_nvme, 'create_hostnqn')
+    @mock.patch.object(utils.priv_nvme, 'create_hostnqn')
     @mock.patch.object(builtins, 'open')
     def test_get_host_nqn_io_err(self, mock_open, mock_create):
         mock_create.return_value = mock.sentinel.nqn
         mock_open.side_effect = IOError()
-        result = self.connector._get_host_nqn()
+        result = utils.get_host_nqn()
         mock_open.assert_called_once_with('/etc/nvme/hostnqn', 'r')
         mock_create.assert_called_once_with()
         self.assertEqual(mock.sentinel.nqn, result)
 
-    @mock.patch.object(nvmeof.priv_nvme, 'create_hostnqn')
+    @mock.patch.object(utils.priv_nvme, 'create_hostnqn')
     @mock.patch.object(builtins, 'open')
     def test_get_host_nqn_err(self, mock_open, mock_create):
         mock_open.side_effect = Exception()
-        result = self.connector._get_host_nqn()
+        result = utils.get_host_nqn()
         mock_open.assert_called_once_with('/etc/nvme/hostnqn', 'r')
         mock_create.assert_not_called()
         self.assertIsNone(result)
