@@ -28,6 +28,7 @@ from os_brick.initiator.connectors import nvmeof
 from os_brick.initiator import linuxfc
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick.tests import base as test_base
+from os_brick import utils
 
 MY_IP = '10.0.0.1'
 FAKE_SCSI_WWN = '1234567890'
@@ -45,7 +46,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
                        return_value=None)
     @mock.patch.object(nvmeof.NVMeOFConnector, '_get_host_uuid',
                        return_value=None)
-    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_host_nqn',
+    @mock.patch.object(utils, 'get_host_nqn',
                        return_value=None)
     @mock.patch.object(iscsi.ISCSIConnector, 'get_initiator',
                        return_value='fakeinitiator')
@@ -106,20 +107,28 @@ class ConnectorUtilsTestCase(test_base.TestCase):
     def test_brick_get_connector_properties(self):
         self._test_brick_get_connector_properties(False, False, False)
 
+    @mock.patch.object(priv_rootwrap, 'custom_execute',
+                       side_effect=OSError(2))
     @mock.patch.object(priv_rootwrap, 'execute', return_value=('', ''))
-    def test_brick_get_connector_properties_multipath(self, mock_execute):
+    def test_brick_get_connector_properties_multipath(self, mock_execute,
+                                                      mock_custom_execute):
         self._test_brick_get_connector_properties(True, True, True)
         mock_execute.assert_called_once_with('multipathd', 'show', 'status',
                                              run_as_root=True,
                                              root_helper='sudo')
+        mock_custom_execute.assert_called_once_with('nvme', 'version')
 
+    @mock.patch.object(priv_rootwrap, 'custom_execute',
+                       side_effect=OSError(2))
     @mock.patch.object(priv_rootwrap, 'execute',
                        side_effect=putils.ProcessExecutionError)
-    def test_brick_get_connector_properties_fallback(self, mock_execute):
+    def test_brick_get_connector_properties_fallback(self, mock_execute,
+                                                     mock_custom_execute):
         self._test_brick_get_connector_properties(True, False, False)
         mock_execute.assert_called_once_with('multipathd', 'show', 'status',
                                              run_as_root=True,
                                              root_helper='sudo')
+        mock_custom_execute.assert_called_once_with('nvme', 'version')
 
     @mock.patch.object(priv_rootwrap, 'execute',
                        side_effect=putils.ProcessExecutionError)
