@@ -10,9 +10,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from oslo_concurrency import processutils as putils
-import six
 
 from os_brick import exception
 from os_brick import privileged
@@ -72,7 +72,15 @@ class PrivRootwrapTestCase(base.TestCase):
                                           run_as_root=False,
                                           keyword_arg=mock.sentinel.kwarg)
 
-    def test_custom_execute(self):
+    @mock.patch('threading.Timer')
+    def test_custom_execute_default_timeout(self, mock_timer):
+        """Confirm timeout defaults to 600 and the thread timer is started."""
+        priv_rootwrap.custom_execute('echo', 'hola')
+        mock_timer.assert_called_once_with(600, mock.ANY, mock.ANY)
+        mock_timer.return_value.start.assert_called_once_with()
+
+    def test_custom_execute_callbacks(self):
+        """Confirm execute callbacks are called on execute."""
         on_execute = mock.Mock()
         on_completion = mock.Mock()
         msg = 'hola'
@@ -85,7 +93,7 @@ class PrivRootwrapTestCase(base.TestCase):
         proc = on_execute.call_args[0][0]
         on_completion.assert_called_once_with(proc)
 
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_custom_execute_timeout_raises_with_retries(self, sleep_mock):
         on_execute = mock.Mock()
         on_completion = mock.Mock()
@@ -105,7 +113,7 @@ class PrivRootwrapTestCase(base.TestCase):
         out, err = priv_rootwrap.custom_execute('sleep', '2', timeout=0.05,
                                                 raise_timeout=False)
         self.assertEqual('', out)
-        self.assertIsInstance(err, six.string_types)
+        self.assertIsInstance(err, str)
 
     def test_custom_execute_check_exit_code(self):
         self.assertRaises(putils.ProcessExecutionError,
@@ -116,7 +124,7 @@ class PrivRootwrapTestCase(base.TestCase):
         out, err = priv_rootwrap.custom_execute('ls', '-y',
                                                 check_exit_code=False)
         self.assertEqual('', out)
-        self.assertIsInstance(err, six.string_types)
+        self.assertIsInstance(err, str)
 
     @mock.patch.object(priv_rootwrap.unlink_root.privsep_entrypoint,
                        'client_mode', False)
